@@ -1,5 +1,5 @@
 """
-AutoWikiAdminMixin
+autoCode
 """
 from django.contrib import admin
 from bddjango.adminclass import BaseAdmin, BulkDeleteMixin
@@ -7,46 +7,8 @@ from bddjango import get_field_names_by_model
 from bddjango.adminclass import BaseAdmin
 from django.http import JsonResponse
 from django.contrib.contenttypes.models import ContentType
-
-
-class AutoWikiAdminMixin:
-    """
-    AutoWikiAdminMixin
-    """
-
-    models = None
-
-    def auto_wiki(self, request, queryset=None, model=None):
-        models = self.models
-        assert models is not None, '必须指定models!'
-
-        from bdtime import Time
-        tt = Time()
-        msg = f'成功建立{1}条索引, 耗时: {tt.now(2)}秒.'
-        self.message_user(request, msg)
-
-        from django.http import FileResponse
-
-        files = 'templates/admin/home.html'
-        file_name = 'test111.txt'
-        response = FileResponse(files)
-        response['content_type'] = "application/octet-stream"
-        response['Content-Disposition'] = 'attachment; filename=' + file_name.encode('utf-8').decode('ISO-8859-1')
-        return response
-
-        # return JsonResponse(data={
-        #     'status': 'success',
-        #     'msg': f'处理成功! 耗时: {tt.now(2)}秒.'
-        # })
-
-    auto_wiki.short_description = "生成Wiki"
-    auto_wiki.icon = 'fa fa-spinner'
-    auto_wiki.confirm = '确定生成Wiki？'
-    auto_wiki.type = 'success'
-    # auto_wiki.layer = {
-    #     'title': '确定生成Wiki?',
-    #     'tips': '该操作可能比较耗时, 请耐心等候...',
-    # }
+from bddjango import convert_query_parameter_to_bool, my_api_assert_function
+from bddjango import find_indexes_in_dc_ls
 
 
 tmp_urls = None
@@ -70,53 +32,6 @@ my_apps = get_my_apps()
 
 
 def get_layer_for_app_and_views():
-    # from django.db import models as m
-    # from django.conf import settings
-    # install_apps = settings.INSTALLED_APPS
-    # print(install_apps)
-
-    # for qs_i in qs_ls:
-    #     if qs_i.app_label == 'activities':
-    #         break
-    # if qs_i.app_label in install_apps:
-    #     break
-    # qs_ls: m.QuerySet
-    # qs_i: ContentType
-
-    # md_i = qs_i.model_class()
-    # model_name = qs_i.model.capitalize()
-    # qs_i.app_label
-    # qs_i.model
-
-    # model_name
-    # app_label = 'activities'
-    # view_name = 'Activity'
-    # url_i = f'/api/{app_label}/{view_name}'
-    # url_i
-    # from activities.views import Activity
-    # from activities import views as tmp
-    # exec(f'global tmp_urls; from {app_label} import urls as tmp_urls;')
-    # tmp_url_i = tmp_urls.urlpatterns[0]
-    # tmp_url_i.__dict__
-    # tmp_url_to_view = tmp_url_i.callback
-    # view_class = tmp_url_to_view.view_class
-    #
-    # func_ls = set()
-    # for tmp_url_i in tmp_urls.urlpatterns:
-    #     _view_class = tmp_url_i.callback.view_class
-    #     if _view_class not in func_ls:
-    #         func_ls.add(_view_class)
-    # view_class
-    # my_apps = []
-
-    # import os
-    # app_path = './'
-    # for app_label in install_apps:
-    #     if os.path.exists(os.path.join(app_path, app_label)):
-    #         my_apps.append(app_label)
-
-    # print(my_apps)
-
     auto_dc_ls = []
     for app_label in my_apps:
         exec(f'global tmp_views; from {app_label} import views as tmp_views;')
@@ -128,20 +43,15 @@ def get_layer_for_app_and_views():
         vs = []
         for f in func_ls:
             md = getattr(views, f)
-            # md = f
             if hasattr(md, 'queryset') and (
                     hasattr(md, 'serializer_class') or hasattr(md, 'list_fields') or hasattr(md,
                                                                                              'retrieve_fields')):
-                # print('---------', f, md)
                 if getattr(md, 'queryset') is not None and (
                         getattr(md, 'serializer_class') or getattr(md, 'list_serializer_class') or getattr(md,
                                                                                                            'auto_generate_serializer_class') or getattr(
                     md, 'retrieve_serializer_class')) or getattr(md, 'list_fields') or getattr(md,
                                                                                                'retrieve_fields'):
-                    # if not view_class_name or f in view_class_name:
                     s += 1
-                    # print(f, md)
-                    # vs.append([f, md])
                     v_i = f'{app_label}.{f}'
                     vs.append(v_i)
         if s != 0:
@@ -150,13 +60,6 @@ def get_layer_for_app_and_views():
             }
             auto_dc_ls.append(dc)
 
-        # from bddjango import my_api_assert_function
-        # my_api_assert_function(s != 0, '没有任何可以转换的view_class! 将自动生成!!')
-
-        # from simpleui.admin import AjaxAdmin
-
-    # from bdtime import show_json, show_ls
-    # show_ls(auto_apps)
     auto_apps = [list(auto_dc.keys())[0] for auto_dc in auto_dc_ls]
 
     default_views = []
@@ -182,7 +85,6 @@ def set_options(ls):
     ContentTypeAdmin.auto_wiki.layer['params'][0]['options'] = options
 
 
-# @admin.register(ContentType)
 class ContentTypeAdmin(BaseAdmin):
     """
     # 自动生成代码和wiki的admin
@@ -197,13 +99,18 @@ class ContentTypeAdmin(BaseAdmin):
     app_exclude = []
 
     list_filter = ['app_label']
-    actions = ['auto_wiki', 'auto_code']
+    actions = ['auto_wiki', 'auto_code', 'auto_model']
 
     default_export_action = False
     custom_import_and_export_buttons = False  # 是否显示自定义的导入导出按钮
     has_import_perm = False  # 导入数据
     has_export_perm = False  # 全部导出
 
+    default_use_complete_model_view = False  # 是否在`auto_code`中默认启用`增删改查`型`view`
+    add_db_column = False  # 是否在`auto_model`中默认增加`db_column`
+
+    codes_template_file_path = None
+
     def get_queryset(self, request):
         total_qs_ls = super().get_queryset(request)
 
@@ -245,24 +152,6 @@ class ContentTypeAdmin(BaseAdmin):
         if _view_class:
             app_label, view_class = _view_class.split('.')
 
-            # from django.test import Client
-            # c = Client()
-            # url = f'/api/{app_label}/{view_class}/'
-            # data = {
-            #     'get_output_file': 1,
-            #     'app_name': app_label,
-            #     'view_class_name': view_class,
-            # }
-
-            # from django.http import HttpRequest
-            # HttpRequest()
-            # type(request)
-            # type(request)
-            # AutoWiki.as_view()(request._request, *args, **kwargs)
-            # request.copy()
-            # from copy import deepcopy
-            # _request = deepcopy(request)
-
             from bddjango.autoWiki import AutoWiki
             from django.test.client import ASGIRequest, FakePayload
             # 伪造一个request请求
@@ -272,8 +161,8 @@ class ContentTypeAdmin(BaseAdmin):
                      'query_string': b'app_label=author',
                      'headers': [(b'host', b'localhost:8000'), (b'connection', b'close'),
                                  (b'content-length', b'624'), (b'accept', b'application/json, text/plain, */*'), (
-                                 b'user-agent',
-                                 b'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36'),
+                                     b'user-agent',
+                                     b'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36'),
                                  (b'content-type',
                                   b'multipart/form-data; boundary=----WebKitFormBoundarym5HMNzwmAdbBBH3J'),
                                  (b'origin', b'http://10.120.65.140:2227'), (b'referer',
@@ -283,8 +172,6 @@ class ContentTypeAdmin(BaseAdmin):
                                                                                     b'fontSize=12; theme=/api/static/admin/simpleui-x/theme/e-blue-pro.css; theme_name=Enterprise%20blue%20pro; fold=false; adminer_key=07406eb2141d4cfdfc1d21fcd7a524fc; adminer_sid=5eb5495ff40b806d32f4b4ad8b4caeb7; MUIDB=3471AAE1567861750DD2BB71571E60CC; ckCsrfToken=VH4uNXIAYS3CG9cfJKHHhfzVj0yxJDwZTsw80Hls; token=dd95c297fe113b65516965ae6f09203c455210fd; UserInfo={%22id%22:12%2C%22username%22:%22test1234%22%2C%22nickname%22:%22%E6%B5%8B%E8%AF%95%E7%94%A8%E6%88%B7%22%2C%22email%22:%22%22%2C%22useriprelation_set%22:[]%2C%22groups%22:[]%2C%22is_staff%22:false%2C%22is_active%22:true%2C%22phone%22:null}; csrftoken=4rtE0HFNmMKiSg0MhKbiGZ976ZyT6NUcSdJGTGAuZ6uMGo2yTV0j8nEeSkJ47r2l; sessionid=eu5muzwony63cwdm9geh8jxm0bhcfi7j; tabstyle=raw-tab')],
                      'client': ['127.0.0.1', 48340], 'server': ['127.0.0.1', 8000], 'asgi': {'version': '3.0'}}
 
-            # from bddjango import show_json
-            # show_json(scope)
             scope.update(
                 {
                     'method': 'GET',
@@ -324,8 +211,7 @@ class ContentTypeAdmin(BaseAdmin):
         })
 
     auto_wiki.short_description = "生成Wiki"
-    auto_wiki.icon = 'fa fa-spinner'
-    # auto_wiki.confirm = '确定生成Wiki？'
+    auto_wiki.icon = 'el-icon-upload'
     auto_wiki.type = 'success'
     auto_wiki.layer = {
         'title': '确定生成Wiki?',
@@ -354,11 +240,8 @@ class ContentTypeAdmin(BaseAdmin):
         ]
     }
 
-    codes_template_file_path = None
-
     def auto_code(self, request, queryset=None, model=None):
         from bdtime import Time
-        from bddjango import convert_query_parameter_to_bool, my_api_assert_function
         from bddjango import get_base_model
         from bddjango.autoWiki import AutoWiki
         import os
@@ -398,14 +281,16 @@ class ContentTypeAdmin(BaseAdmin):
         checkbox = [i.replace('.py', '') for i in checkbox]
 
         _codes_template_file_path = AutoWiki.path_of_jinja2_template
-        _codes_template_file_path = os.path.join(os.path.dirname(replace_path_by_platform(AutoWiki.path_of_jinja2_template)), 'autoCode.html')
+        _codes_template_file_path = os.path.join(
+            os.path.dirname(replace_path_by_platform(AutoWiki.path_of_jinja2_template)), 'autoCode.html')
 
         codes_template_file_path = self.codes_template_file_path if self.codes_template_file_path else _codes_template_file_path
-        my_api_assert_function(convert_query_parameter_to_bool(codes_template_file_path), f'`codes_template_file_path`不能为空!')
+        my_api_assert_function(convert_query_parameter_to_bool(codes_template_file_path),
+                               f'`codes_template_file_path`不能为空!')
         codes_template_file_path = replace_path_by_platform(codes_template_file_path)
-        my_api_assert_function(os.path.exists(codes_template_file_path), f'`codes_template_file_path`路径不存在![{codes_template_file_path}]')
+        my_api_assert_function(os.path.exists(codes_template_file_path),
+                               f'`codes_template_file_path`路径不存在![{codes_template_file_path}]')
 
-        # print('--- res_context_dc:', res_context_dc)
 
         # region # --- 开始填充jinja模板
         from jinja2 import Template
@@ -413,13 +298,6 @@ class ContentTypeAdmin(BaseAdmin):
         with open(codes_template_file_path, encoding='utf-8') as file_:
             template = Template(file_.read())
 
-        # from jinja2 import PackageLoader, Environment, FileSystemLoader
-        # env = Environment(loader=PackageLoader('python_project', 'templates'))  # 创建一个包加载器对象
-
-        # env = Environment(loader=FileSystemLoader(app_name))  # 文件加载器, 可用`list_templates`方法查看存在哪些东西
-        # os.path.exists(path_of_jinja2_template)
-        # template = env.get_template(path_of_jinja2_template)
-
         tempdir_rootpath = 'media/tempdir'  # 临时输出文件的根目录
         output_dirpath = os.path.join(tempdir_rootpath, 'output')  # 临时输出文件的子目录
         os.makedirs(output_dirpath, exist_ok=True)
@@ -429,7 +307,6 @@ class ContentTypeAdmin(BaseAdmin):
         object_name = md_i._meta.object_name
         verbose_name = md_i._meta.verbose_name
 
-        # db_table = md_i._meta.db_table
         output_fname = 'codes__' + app_label + '_' + object_name + output_f_suffix
 
         output_fpath = os.path.join(output_dirpath, output_fname)
@@ -437,7 +314,6 @@ class ContentTypeAdmin(BaseAdmin):
             os.remove(output_fpath)
 
         _fields = get_field_names_by_model(md_i, exclude_fields=['id'])
-        # _fields = get_field_names_by_model(md_i)
 
         field_dc_ls = []
         field_dc_i = {}
@@ -447,7 +323,7 @@ class ContentTypeAdmin(BaseAdmin):
             # break
             f_type = get_field_type_in_py(md_i, f_name)
             if f_type in ['int', 'str']:
-                f_value = 1 if f_type == 'int' else '"xxx"'       # 样例数据
+                f_value = 1 if f_type == 'int' else '"xxx"'  # 样例数据
                 field_dc_i.update(
                     {
                         'f_name': f_name,
@@ -505,7 +381,7 @@ class ContentTypeAdmin(BaseAdmin):
         })
 
     auto_code.short_description = "代码生成"
-    auto_code.icon = 'fa fa-spinner'
+    auto_code.icon = 'el-icon-upload'
     auto_code.type = 'success'
     auto_code.layer = {
         'title': '确定生成代码?',
@@ -541,27 +417,155 @@ class ContentTypeAdmin(BaseAdmin):
                 'type': 'switch',
                 'key': 'use_complete_model_view',
                 'label': '增删改查',
-                'value': True
+                'value': default_use_complete_model_view
             },
-            # {
-            #     # 这里的type 对应el-input的原生input属性，默认为input
-            #     'type': 'select',
-            #     # key 对应post参数中的key
-            #     'key': 'use_complete_model_view',
-            #     # 显示的文本
-            #     'label': '增删改查',
-            #     'value': '0',
-            #     # 'parent_key': 'app_label',
-            #     'options': [
-            #         {
-            #             'key': '0',
-            #             'label': '否'
-            #         },
-            #         {
-            #             'key': '1',
-            #             'label': '是'
-            #         }
-            #     ]
-            # },
         ]
     }
+
+    def auto_model(self, request, queryset):
+        from bddjango.tools.generateModelCodeFromExcel import get_model_info
+        import os
+        from bdtime import tt
+        import xlrd
+        import pandas as pd
+        from bddjango import convert_query_parameter_to_bool
+
+        format_ls = ['xls', 'xlsx', 'csv']
+
+        upload = request.FILES.get('upload')
+        if not upload:
+            return JsonResponse(data={
+                'status': 'error',
+                'msg': f'上传文件不能为空! 支持的文件格式为: {format_ls}'
+            })
+
+        f_format = upload._name.rsplit('.', 1)[-1]
+        if f_format not in format_ls:
+            return JsonResponse(data={
+                'status': 'error',
+                'msg': f'不支持的文件类型! 目前仅支持{format_ls}.'
+            })
+
+        post = request.POST
+        add_db_column = post.get('add_db_column')
+        add_db_column = convert_query_parameter_to_bool(add_db_column)
+        print(f'~~~ add_db_column: {add_db_column}')
+
+        tempdir = 'tempdir/auto_model'
+        os.makedirs(tempdir, exist_ok=True)
+
+        file_contents = upload.file.read()
+        base_name, _ = os.path.splitext(upload._name)
+
+        fname = upload._name
+        f_path = os.path.join(tempdir, fname)
+
+        if f_format == 'csv':
+            try:
+                encoding = 'utf-8'
+                file_data = file_contents.decode(encoding)
+            except Exception as e:
+                print('-- 尝试用gbk编码 --')
+                encoding = 'gbk'
+                file_data = file_contents.decode(encoding)
+
+            with open(f_path, 'w', encoding=encoding) as f:
+                f.write(file_data)
+
+            # # 为解决字段内有逗号导致分割错误问题, 只能采用pd了
+            # df = pd.read_csv(fname, encoding=encoding)
+        elif f_format == 'xls':
+            wb = xlrd.open_workbook(file_contents=file_contents)
+            df = pd.read_excel(wb)
+            df.to_excel(f_path, index=False, encoding='utf-8')
+        elif f_format == 'xlsx':
+            try:
+                wb = xlrd.open_workbook(file_contents=file_contents)
+                df = pd.read_excel(wb)
+                df.to_excel(f_path, index=False, encoding='utf-8')
+            except Exception as e:
+                raise TypeError(f'文件读入错误! 可转换为`xls`格式后重试. msg: {e}')
+
+        model_info = get_model_info(f_path, add_db_column=add_db_column)
+        content = model_info.get('model_code')
+        output_fname = model_info.get('model_name')
+
+        tempdir_rootpath = 'media/tempdir'  # 临时输出文件的根目录
+        output_dirpath = os.path.join(tempdir_rootpath, 'output')  # 临时输出文件的子目录
+        output_fpath = os.path.join(output_dirpath,
+                                    f'{output_fname}_{tt.get_current_beijing_time_str(tt.common_date_time_formats.ms_int)}.txt')
+        with open(output_fpath, 'w', encoding='utf-8') as f:
+            f.write(content)
+
+        print('\n---------- auto_model:')
+        print(content)
+
+        msg = f'成功生成模型代码, 耗时: {tt.now(2)}秒. <br> <a href="/api/{output_fpath}" target="_blank">点击此处查看</a>'
+        self.message_user(request, msg)
+
+        from bddjango import remove_temp_file
+        remove_temp_file(tempdir, MAX_TEMPS=20, desc='--- tempdir')
+        remove_temp_file(output_dirpath, MAX_TEMPS=20, desc='--- output_dirpath')
+
+        return JsonResponse(data={
+            'status': 'success',
+            'msg': f'处理成功! 耗时: {tt.now(2)}秒.'
+        })
+
+    auto_model.short_description = '模型生成'
+    auto_model.type = 'success'
+    auto_model.icon = 'el-icon-upload'
+    auto_model.enable = True
+
+    auto_model.layer = {
+        'params': [
+            {
+                'type': 'file',
+                'key': 'upload',
+                'label': '数据文件'
+            },
+            {
+                'type': 'switch',
+                'key': 'add_db_column',
+                'label': 'db列名',
+                'value': add_db_column
+            },
+        ]
+    }
+
+
+def register_content_type_admin(app_filter=None, app_exclude=None, list_filter=None,
+                                default_use_complete_model_view=None, add_db_column=None):
+    """
+    自动注册content_type_admin
+
+    :param app_filter: app过滤列表
+    :param app_exclude: app排除列表
+    :param list_filter: 过滤字段
+    :param default_use_complete_model_view: 是否在`auto_code`中默认启用`增删改查`型`view`
+    :param add_db_column: 是否在`auto_model`中默认增加`db_column`
+
+    --- eg:
+    >>>　register_content_type_admin(default_use_complete_model_view=1, add_db_column=0)
+    """
+    if app_filter is not None:
+        ContentTypeAdmin.app_filter = app_filter
+    if app_exclude is not None:
+        ContentTypeAdmin.app_exclude = app_exclude
+    if list_filter is not None:
+        ContentTypeAdmin.list_filter = list_filter
+    if default_use_complete_model_view is not None:
+        default_use_complete_model_view = convert_query_parameter_to_bool(default_use_complete_model_view)
+        ContentTypeAdmin.default_use_complete_model_view = default_use_complete_model_view
+        params = ContentTypeAdmin.auto_code.layer['params']
+        indexes = find_indexes_in_dc_ls(params, 'key', 'use_complete_model_view')
+        assert len(indexes) == 1, 'ContentTypeAdmin.layer重置`use_complete_model_view`出错'
+        params[indexes[0]]['value'] = default_use_complete_model_view
+    if add_db_column is not None:
+        add_db_column = convert_query_parameter_to_bool(add_db_column)
+        ContentTypeAdmin.add_db_column = add_db_column
+        params = ContentTypeAdmin.auto_model.layer['params']
+        indexes = find_indexes_in_dc_ls(params, 'key', 'add_db_column')
+        assert len(indexes) == 1, 'ContentTypeAdmin.layer重置`add_db_column`出错'
+        params[indexes[0]]['value'] = add_db_column
+    admin.site.register(ContentType, ContentTypeAdmin)
